@@ -64,22 +64,23 @@ describe('Validation errors', function () {
         , invalidData = { foo: 1, bar: 2, baz: 3, quux: 4 };
 
       var path = pathFunc(errorDataPath);
+      var msg = additionalFunc(errorDataPath);
 
       var validate = ajv.compile(schema);
       shouldBeValid(validate, data);
       shouldBeInvalid(validate, invalidData);
-      shouldBeError(validate.errors[0], 'additionalProperties', '#/additionalProperties', path("['baz']"), undefined, { additionalProperty: 'baz' });
+      shouldBeError(validate.errors[0], 'additionalProperties', '#/additionalProperties', path("['baz']"), msg, { additionalProperty: 'baz' });
 
       var validateJP = ajvJP.compile(schema);
       shouldBeValid(validateJP, data);
       shouldBeInvalid(validateJP, invalidData);
-      shouldBeError(validateJP.errors[0], 'additionalProperties', '#/additionalProperties', path("/baz"), undefined, { additionalProperty: 'baz' });
+      shouldBeError(validateJP.errors[0], 'additionalProperties', '#/additionalProperties', path("/baz"), msg, { additionalProperty: 'baz' });
 
       var fullValidate = fullAjv.compile(schema);
       shouldBeValid(fullValidate, data);
       shouldBeInvalid(fullValidate, invalidData, 2);
-      shouldBeError(fullValidate.errors[0], 'additionalProperties', '#/additionalProperties', path('/baz'), undefined, { additionalProperty: 'baz' });
-      shouldBeError(fullValidate.errors[1], 'additionalProperties', '#/additionalProperties', path('/quux'), undefined, { additionalProperty: 'quux' });
+      shouldBeError(fullValidate.errors[0], 'additionalProperties', '#/additionalProperties', path('/baz'), msg, { additionalProperty: 'baz' });
+      shouldBeError(fullValidate.errors[1], 'additionalProperties', '#/additionalProperties', path('/quux'), msg, { additionalProperty: 'quux' });
 
       if (errorDataPath == 'property') {
         fullValidate.errors
@@ -184,7 +185,7 @@ describe('Validation errors', function () {
       delete invalidData2[98];
 
       var path = pathFunc(errorDataPath);
-      var msg = msgFunc(errorDataPath);
+      var msg = requiredFunc(errorDataPath);
 
       test();
 
@@ -368,7 +369,7 @@ describe('Validation errors', function () {
       , invalidData2 = { bar: 2 };
 
     var path = pathFunc(errorDataPath);
-    var msg = msgFunc(errorDataPath);
+    var msg = requiredFunc(errorDataPath);
 
     var validate = ajv.compile(schema);
     shouldBeValid(validate, data);
@@ -399,7 +400,7 @@ describe('Validation errors', function () {
     };
   }
 
-  function msgFunc(errorDataPath) {
+  function requiredFunc(errorDataPath) {
     return function (prop) {
       return errorDataPath == 'property'
               ? 'is a required property'
@@ -407,10 +408,16 @@ describe('Validation errors', function () {
     };
   }
 
+  function additionalFunc(errorDataPath) {
+    return errorDataPath == 'property'
+            ? 'is an invalid additional property'
+            : 'should NOT have additional properties';
+  }
+
 
   it('"items" errors should include item index without quotes in dataPath (#48)', function() {
     var schema1 = {
-      id: 'schema1',
+      $id: 'schema1',
       type: 'array',
       items: {
         type: 'integer',
@@ -445,7 +452,7 @@ describe('Validation errors', function () {
     shouldBeError(fullValidate.errors[1], 'minimum', '#/items/minimum', '/3', 'should be >= 10');
 
     var schema2 = {
-      id: 'schema2',
+      $id: 'schema2',
       type: 'array',
       items: [{ minimum: 10 }, { minimum: 9 }, { minimum: 12 }]
     };
@@ -539,6 +546,39 @@ describe('Validation errors', function () {
         validate(1.5) .should.equal(true);
       }
     });
+
+    it('should return passing schemas in error params', function() {
+      var schema = {
+        oneOf: [
+          { type: 'number' },
+          { type: 'integer' },
+          { const: 1.5 }
+        ]
+      };
+
+      test(ajv);
+      test(fullAjv);
+
+      function test(_ajv) {
+        var validate = _ajv.compile(schema);
+        validate(1) .should.equal(false);
+        var err = validate.errors.pop();
+        err.keyword .should.equal('oneOf');
+        err.params .should.eql({passingSchemas: [0, 1]});
+
+        validate(1.5) .should.equal(false);
+        err = validate.errors.pop();
+        err.keyword .should.equal('oneOf');
+        err.params .should.eql({passingSchemas: [0, 2]});
+
+        validate(2.5) .should.equal(true);
+
+        validate('foo') .should.equal(false);
+        err = validate.errors.pop();
+        err.keyword .should.equal('oneOf');
+        err.params .should.eql({passingSchemas: null});
+      }
+    });
   });
 
 
@@ -565,6 +605,257 @@ describe('Validation errors', function () {
   });
 
 
+  describe('type errors', function() {
+    describe('integer', function() {
+      it('should have only one error in {allErrors: false} mode', function() {
+        test(ajv);
+      });
+
+      it('should return all errors in {allErrors: true} mode', function() {
+        test(fullAjv, 2);
+      });
+
+      function test(_ajv, numErrors) {
+        var schema = {
+          type: 'integer',
+          minimum: 5
+        };
+
+
+        var validate = _ajv.compile(schema);
+        shouldBeValid(validate, 5);
+        shouldBeInvalid(validate, 5.5);
+        shouldBeInvalid(validate, 4);
+        shouldBeInvalid(validate, '4');
+        shouldBeInvalid(validate, 4.5, numErrors);
+      }
+    });
+
+    describe('keyword for another type', function() {
+      it('should have only one error in {allErrors: false} mode', function() {
+        test(ajv);
+      });
+
+      it('should return all errors in {allErrors: true} mode', function() {
+        test(fullAjv, 2);
+      });
+
+      function test(_ajv, numErrors) {
+        var schema = {
+          type: 'array',
+          minItems: 2,
+          minimum: 5
+        };
+
+
+        var validate = _ajv.compile(schema);
+        shouldBeValid(validate, [1, 2]);
+        shouldBeInvalid(validate, [1]);
+        shouldBeInvalid(validate, 5);
+        shouldBeInvalid(validate, 4, numErrors);
+      }
+    });
+
+    describe('array of types', function() {
+      it('should have only one error in {allErrors: false} mode', function() {
+        test(ajv);
+      });
+
+      it('should return all errors in {allErrors: true} mode', function() {
+        test(fullAjv, 2);
+      });
+
+      function test(_ajv, numErrors) {
+        var schema = {
+          type: ['array', 'object'],
+          minItems: 2,
+          minProperties: 2,
+          minimum: 5
+        };
+
+
+        var validate = _ajv.compile(schema);
+        shouldBeValid(validate, [1, 2]);
+        shouldBeValid(validate, {foo: 1, bar: 2});
+        shouldBeInvalid(validate, [1]);
+        shouldBeInvalid(validate, {foo: 1});
+        shouldBeInvalid(validate, 5);
+        shouldBeInvalid(validate, 4, numErrors);
+      }
+    });
+  });
+
+
+  describe('exclusiveMaximum/Minimum errors', function() {
+    it('should include limits in error message', function() {
+      var schema = {
+        type: 'integer',
+        exclusiveMinimum: 2,
+        exclusiveMaximum: 5
+      };
+
+      [ajv, fullAjv].forEach(function (_ajv) {
+        var validate = _ajv.compile(schema);
+        shouldBeValid(validate, 3);
+        shouldBeValid(validate, 4);
+
+        shouldBeInvalid(validate, 2);
+        testError('exclusiveMinimum', 'should be > 2', {comparison: '>', limit: 2, exclusive: true});
+
+        shouldBeInvalid(validate, 5);
+        testError('exclusiveMaximum', 'should be < 5', {comparison: '<', limit: 5, exclusive: true});
+
+        function testError(keyword, message, params) {
+          var err = validate.errors[0];
+          shouldBeError(err, keyword, '#/' + keyword, '', message, params);
+        }
+      });
+    });
+
+    it('should include limits in error message with $data', function() {
+      var schema = {
+        "properties": {
+          "smaller": {
+            "type": "number",
+            "exclusiveMaximum": { "$data": "1/larger" }
+          },
+          "larger": { "type": "number" }
+        }
+      };
+
+      ajv = new Ajv({$data: true});
+      fullAjv = new Ajv({$data: true, allErrors: true, verbose: true, jsonPointers: true});
+
+      [ajv, fullAjv].forEach(function (_ajv) {
+        var validate = _ajv.compile(schema);
+        shouldBeValid(validate, {smaller: 2, larger: 4});
+        shouldBeValid(validate, {smaller: 3, larger: 4});
+
+        shouldBeInvalid(validate, {smaller: 4, larger: 4});
+        testError();
+
+        shouldBeInvalid(validate, {smaller: 5, larger: 4});
+        testError();
+
+        function testError() {
+          var err = validate.errors[0];
+          shouldBeError(err, 'exclusiveMaximum',
+                        '#/properties/smaller/exclusiveMaximum',
+                        _ajv._opts.jsonPointers ? '/smaller' : '.smaller',
+                        'should be < 4',
+                        {comparison: '<', limit: 4, exclusive: true});
+        }
+      });
+    });
+  });
+
+
+  describe('if/then/else errors', function() {
+    var validate, numErrors;
+
+    it('if/then/else should include failing keyword in message and params', function() {
+      var schema = {
+        'if': { maximum: 10 },
+        'then': { multipleOf: 2 },
+        'else': { multipleOf: 5 }
+      };
+
+      [ajv, fullAjv].forEach(function (_ajv) {
+        prepareTest(_ajv, schema);
+        shouldBeValid(validate, 8);
+        shouldBeValid(validate, 15);
+
+        shouldBeInvalid(validate, 7, numErrors);
+        testIfError('then', 2);
+
+        shouldBeInvalid(validate, 17, numErrors);
+        testIfError('else', 5);
+      });
+    });
+
+    it('if/then should include failing keyword in message and params', function() {
+      var schema = {
+        'if': { maximum: 10 },
+        'then': { multipleOf: 2 }
+      };
+
+      [ajv, fullAjv].forEach(function (_ajv) {
+        prepareTest(_ajv, schema);
+        shouldBeValid(validate, 8);
+        shouldBeValid(validate, 11);
+        shouldBeValid(validate, 12);
+
+        shouldBeInvalid(validate, 7, numErrors);
+        testIfError('then', 2);
+      });
+    });
+
+    it('if/else should include failing keyword in message and params', function() {
+      var schema = {
+        'if': { maximum: 10 },
+        'else': { multipleOf: 5 }
+      };
+
+      [ajv, fullAjv].forEach(function (_ajv) {
+        prepareTest(_ajv, schema);
+        shouldBeValid(validate, 7);
+        shouldBeValid(validate, 8);
+        shouldBeValid(validate, 15);
+
+        shouldBeInvalid(validate, 17, numErrors);
+        testIfError('else', 5);
+      });
+    });
+
+    function prepareTest(_ajv, schema) {
+      validate = _ajv.compile(schema);
+      numErrors = _ajv._opts.allErrors ? 2 : 1;
+    }
+
+    function testIfError(ifClause, multipleOf) {
+      var err = validate.errors[0];
+      shouldBeError(err, 'multipleOf', '#/' + ifClause + '/multipleOf', '',
+        'should be multiple of ' + multipleOf, {multipleOf: multipleOf});
+
+      if (numErrors == 2) {
+        err = validate.errors[1];
+        shouldBeError(err, 'if', '#/if', '',
+          'should match "' + ifClause + '" schema', {failingKeyword: ifClause});
+      }
+    }
+  });
+
+
+  describe('uniqueItems errors', function() {
+    it('should not return uniqueItems error when non-unique items are of a different type than required', function() {
+      var schema = {
+        items: {type: 'number'},
+        uniqueItems: true
+      };
+
+      [ajvJP, fullAjv].forEach(function (_ajv) {
+        var validate = _ajv.compile(schema);
+        shouldBeValid(validate, [1, 2, 3]);
+
+        shouldBeInvalid(validate, [1, 2, 2]);
+        shouldBeError(validate.errors[0], 'uniqueItems', '#/uniqueItems', '',
+          'should NOT have duplicate items (items ## 2 and 1 are identical)',
+          {i: 1, j: 2});
+
+        var expectedErrors = _ajv._opts.allErrors ? 2 : 1;
+        shouldBeInvalid(validate, [1, "2", "2", 2], expectedErrors);
+        testTypeError(0, '/1');
+        if (expectedErrors == 2) testTypeError(1, '/2');
+
+        function testTypeError(i, dataPath) {
+          var err = validate.errors[i];
+          shouldBeError(err, 'type', '#/items/type', dataPath, 'should be number');
+        }
+      });
+    });
+  });
+
+
   function testSchema1(schema, schemaPathPrefix) {
     _testSchema1(ajv, schema, schemaPathPrefix);
     _testSchema1(ajvJP, schema, schemaPathPrefix);
